class BaseConverter(object):
    """
    Convert numbers from base 10 integers to base X strings and back again.
    The default converts to/from base62 strings.
    
    Based on http://www.djangosnippets.org/snippets/1431/
    """
    decimal_digits = '0123456789'
    base62_digits = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789abcdefghijklmnopqrstuvwxyz'
    
    def __init__(self, digits=base62_digits):
        self.digits = digits
    
    def from_decimal(self, i):
        return self.convert(i, self.decimal_digits, self.digits)
    
    def to_decimal(self, s):
        return int(self.convert(s, self.digits, self.decimal_digits))
    
    def convert(number, fromdigits, todigits):
        # Based on http://code.activestate.com/recipes/111286/
        if str(number)[0] == '-':
            number = str(number)[1:]
            neg = 1
        else:
            neg = 0
 
        # make an integer out of the number
        x = 0
        for digit in str(number):
           x = x * len(fromdigits) + fromdigits.index(digit)
    
        # create the result in base 'len(todigits)'
        if x == 0:
            res = todigits[0]
        else:
            res = ""
            while x > 0:
                digit = x % len(todigits)
                res = todigits[digit] + res
                x = int(x / len(todigits))
            if neg:
                res = '-' + res
        return res
    convert = staticmethod(convert)
 



import datetime

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponse, HttpResponseBadRequest
from boto.s3.connection import S3Connection
from boto.s3.key import Key

from scrup.models import *
from scrup.utils import BaseConverter

key = getattr(settings, 'SCRUP_AWS_ACCESS_KEY', None)
secret = getattr(settings, 'SCRUP_AWS_SECRET_KEY', None)
bucket = getattr(settings, 'SCRUP_AWS_BUCKET', None)
prefix = getattr(settings, 'SCRUP_AWS_PREFIX', '')
cname = getattr(settings, 'SCRUP_AWS_CNAME', False)

url_template = 'http://%s/%s' % (
    bucket if cname else bucket + ".s3.amazonaws.com",
    prefix + '/' if prefix else '')

if key and secret and bucket:
    # TODO: exception handling
    conn = S3Connection(key, secret)
else:
    raise ImproperlyConfigured('django-scrup AWS config requires key, secret, and bucket.')

base62_converter = BaseConverter()


def upload(request, filename=None):
    if request.method != 'POST':
        return HttpResponseBadRequest()
    now = datetime.datetime.now()
    today = now.date()
    d, created = Date.objects.get_or_create(date=today)
    
    s = Screenshot.objects.create(date=d, name=filename, timestamp=now)
    print s.id
    converted = base62_converter.from_decimal(s.id)
    url = url_template + converted
    print url
    
    # TODO: exception handling
    headers = {
        'Content-Type': 'image/png',
    }
    b = conn.create_bucket(bucket)
    k = Key(b)
    k.key = converted
    # Hacky? Can't think of a better way to do this.
    k.set_contents_from_string(request.raw_post_data, headers=headers, policy='public-read')
    
    response = HttpResponse()
    response.status_code = 201
    response.content = url
    return response


import datetime

from django.db import models

class Date(models.Model):
    """A date during which a screenshot was uploaded"""
    date = models.DateField(default=datetime.date.today)
    
    def __unicode__(self):
        return self.date.isoformat()
    
class Screenshot(models.Model):
    """A single screenshot uploaded to django-scrup."""
    date = models.ForeignKey(Date, related_name='screenshots')
    name = models.CharField(blank=True, null=True, max_length=255)
    timestamp = models.DateTimeField(default=datetime.datetime.now)
    
    class Meta:
        ordering = ('-timestamp',)
        verbose_name, verbose_name_plural = "Screenshot", "Screenshots"

    def __unicode__(self):
        return self.name or self.timestamp.isoformat()



from setuptools import setup, find_packages
 
f = open('README.rst')
readme = f.read()
f.close()
 
setup(
    name = 'django-scrup',
    version = '0.1',
    description = 'A django-based web receiver for Scrup which stores screencaptures on S3.',
    packages = find_packages(),
    long_description = readme,
    author = 'Idan Gazit',
    author_email = 'idan@pixane.com',
    url = 'http://github.com/idangazit/django-scrup',
    install_requires = ('boto>=1.9b',),
    classifiers = (
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ),
)
